'''
Aufgabe 1 (b)
'''
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import numpy as np
import time
import pgm



def frequencyDomainCorrelation(image, model):
    
    # Durschnittswerte berechnen:
    modelMean = np.mean(model)
    imageMean = np.mean(image)
    
    # Bild und Modell zentrieren (Durchschnittswerte abziehen):
    model = model - modelMean      
    image = image - imageMean
       
    # Padding hinzufuegen:
    imH, imW = image.shape
    mH, mW = model.shape
    modelPadded = np.zeros((2*imH, 2*imW))
    modelPadded[0:mH, 0:mW] = model
    imagePadded = np.zeros((2*imH, 2*imW))
    imagePadded[0:imH, 0:imW] = image
    
    # FFTs berechnen:
    imageFT = np.fft.fft2(imagePadded)
    modelFT = np.fft.fft2(modelPadded)
          
    # Korrelation berechnen:
    result = imageFT * np.conjugate(modelFT) 
    # Mit komplex Konjugierter ^ da Korrelation (nicht Konvolution)  
      
    result = np.fft.ifft2(result) # Ruecktransformation
    result = result[0:imH, 0:imW] # Padding entfernen
           
    return result
    


def aufgabe1b():
    
    image = pgm.imread("shelf.pgm")
    model = pgm.imread("shelf-model1.pgm")
    
    start = time.time()
    result = frequencyDomainCorrelation(image, model)
    end = time.time()
    print "Time taken to compute Correlation:", end - start
    
    result = result.real
    
    resultThreshold = result.copy()
    resultThreshold[ resultThreshold < 0.75 * np.max(resultThreshold)] = 0
    
    plt.figure(1)
    plt.subplot(221)
    plt.imshow(model, cmap=plt.cm.gray, norm=colors.NoNorm())
    plt.subplot(222)
    plt.imshow(image, cmap=plt.cm.gray, norm=colors.NoNorm())
    plt.subplot(223)
    plt.imshow(result, cmap=plt.cm.jet)
    plt.subplot(224)
    plt.imshow(resultThreshold, cmap=plt.cm.jet)
    plt.show()
    
    

aufgabe1b()



''' 
[Textaufgabe] Vergleich der Verfahren:

Die bemerkenswertesten Unterschiede zwischen der Ortsraum- und Frequenzraum-Korrelation
liegen in der Rechendauer (Frequenzraum ist deutlich schneller) und der Qualitaet der
"Suche". Hier finde ich die Ortsraum-Korrelation auf den gegebenen Beispielbildern
deutlich besser, da sie selektiver ist und immer tatsaechlich (fast) nur den korrekten
Bereich findet.

'''





